<?php

class IBX_WPFomo_ConvertKit_Helper {
	/**
	 * @since 1.0.0
	 * @var object $api_instance
	 * @access private
	 */
	static private $api_instance = null;

	/**
	 * Get an instance of the API.
	 *
	 * @since 1.0.0
	 * @param string $api_key A valid API key.
	 * @return object The API instance.
	 */
	static public function get_api( $api_key )
	{
		if ( self::$api_instance ) {
			return self::$api_instance;
		}
		if ( ! class_exists( 'ConvertKit' ) ) {
			require_once IBX_WPFOMO_CONVERTKIT_DIR . 'includes/vendor/convertkit/ConvertKit.php';
		}

		self::$api_instance = new ConvertKit( $api_key );

		return self::$api_instance;
	}

    /**
	 * Test the API connection.
	 *
	 * @since 1.0.0
	 * @param array $fields {
	 *      @type string $api_key A valid API key.
	 * }
	 * @return array{
	 *      @type bool|string $error The error message or false if no error.
	 *      @type array $data An array of data used to make the connection.
	 * }
	 */
	static public function connect( $fields = array() )
	{
		$response = array(
			'error'  => false,
			'data'   => array(),
		);

		// Make sure we have an API key.
		if ( ! isset( $fields['api_key'] ) || empty( $fields['api_key'] ) ) {
			$response['error'] = __( 'Error: You must provide an API key.', 'ibx-wpfomo' );
		} // Try to connect and store the connection data.
		else {
			try {
				$api = self::get_api( $fields['api_key'] );

				if ( $api->is_authenticated() ) {
					$response['data'] = array(
						'api_key' => $fields['api_key'],
					);
				} else {
					$response['error'] = sprintf( __( 'Error: Please check your API key. %s', 'ibx-wpfomo' ), $api::$response['error_message'] );
				}

			} catch ( Exception $e ) {
				$response['error'] = $e->getMessage();
			}
		}

		return $response;
	}

    static public function get_members( $args = array() )
    {
        $response = array(
			'error'      => false,
			'members'    => array()
		);

        // Make sure we have an API key.
		if ( ! isset( $args['api_key'] ) || empty( $args['api_key'] ) ) {
			$response['error'] = __( 'Error: You must provide an API key.', 'ibx-wpfomo' );
		}
		// Make sure we have an API secret.
		elseif ( ! isset( $args['api_secret'] ) || empty( $args['api_secret'] ) ) {
			$response['error'] = __( 'Error: You must provide an API Secret.', 'ibx-wpfomo' );
		}
		// Make sure we have form id.
		elseif ( ! isset( $args['form_id'] ) || empty( $args['form_id'] ) ) {
			$response['error'] = __( 'Error: You must provide a form ID.', 'ibx-wpfomo' );
		}
		else {
			$api = self::get_api( $args['api_key'] );

            try {
                $members = $api->make_request('v3/forms/' . $args['form_id'] . '/subscriptions', 'GET', array(
					'api_secret' 		=> $args['api_secret'],
					'sort_order'		=> 'desc',
					'subscriber_state' 	=> 'active'
				));

                $response['members'] = $members;
            }
            catch ( Exception $e ) {
                $response['error'] = $e->getMessage();
            }
		}

        return $response;
    }
}
