<?php
/*
 * Plugin Name: WPfomify - ConvertKit Add-on
 * Plugin URI: https://wpfomify.com
 * Version: 1.0.0
 * Description: ConvertKit Add-on for WPfomify - The Social Proof Marketing plugin for WordPress
 * Author: IdeaBox Creations
 * Author URI: https://ideaboxcreations.com
 * Copyright: (c) 2017 IdeaBox Creations
 * License: GNU General Public License v2.0
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: ibx-wpfomo
 */

define( 'IBX_WPFOMO_CONVERTKIT_VER', '1.0.0' );
define( 'IBX_WPFOMO_CONVERTKIT_DIR', plugin_dir_path( __FILE__ ) );
define( 'IBX_WPFOMO_CONVERTKIT_URL', plugins_url( '/', __FILE__ ) );
define( 'IBX_WPFOMO_CONVERTKIT_PATH', plugin_basename( __FILE__ ) );

class IBX_WPFomo_ConvertKit {
    /**
     * Holds the class object.
     *
     * @since 1.0.0
     * @var object
     */
    public static $instance;

    /**
     * Holds the error messages to render via admin_notices.
     *
     * @since 1.0.0
     * @var array $errors
     */
	public $errors = array();
	
	/**
     * Holds the meta key for members data.
     *
     * @since 1.0.0
     * @var string $meta_key
     */
	public $meta_key = 'convertkit_members';

    /**
     * Primary class constructor.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        add_action( 'plugins_loaded', array( $this, 'init_hooks' ) );
		add_action( 'admin_init', array( $this, 'plugin_updater' ), 0 );
		add_filter( 'ibx_wpfomo_update_meta_once', array( $this, 'update_meta' ), 10, 1 );
		add_filter( 'ibx_wpfomo_admin_general_settings', array( $this, 'register_admin_settings' ), 10, 1 );
		add_action( 'ibx_wpfomo_cron_update_data', array( $this, 'update_data' ), 10, 1 );
    }

    /**
     * Initialize hooks.
     *
     * @since 1.0.0
     */
    public function init_hooks()
    {
		if ( ! class_exists( 'IBX_WPFomo_Loader' ) ) {
			$this->errors[] = __('Please install and activate WPfomify plugin to use WPfomify - ConvertKit add-on.', 'bb-powerpack');
			add_action( 'admin_notices', array( $this, 'admin_notices' ) );
			add_action( 'network_admin_notices', array( $this, 'admin_notices' ) );
			return;
		}
		
		require_once IBX_WPFOMO_CONVERTKIT_DIR . 'classes/class-ibx-wpfomo-convertkit.php';
		
		add_action( 'admin_notices', array( $this, 'admin_notices' ) );
		add_action( 'network_admin_notices', array( $this, 'admin_notices' ) );
        add_action( 'ibx_wpfomo_before_metabox_load', array( $this, 'init_fields' ) );
        add_action( 'ibx_wpfomo_admin_meta_scripts', array( $this, 'enqueue_scripts' ) );
        add_action( 'ibx_wpfomo_admin_settings_scripts', array( $this, 'enqueue_scripts' ) );
        add_filter( 'ibx_wpfomo_conversion_data', array( $this, 'add_conversion_data' ), 10, 2 );
        add_action( 'wp_ajax_ibx_wpfomo_connect_convertkit', array( $this, 'connect_convertkit' ) );
        add_action( 'ibx_wpfomo_save_post', array( $this, 'save_post' ), 10, 1 );
    }

    /**
     * Plugin updater.
     *
     * @since 1.0.0
     */
    public function plugin_updater() {

        if ( ! class_exists( 'IBX_WPFomo_Plugin_Updater' ) ) {
            return;
        }

    	$license_key = trim( get_option( 'ibx_wpfomo_license_key' ) );

    	// setup the updater
    	$updater = new IBX_WPFomo_Plugin_Updater( IBX_WPFOMO_SL_URL, __FILE__, array(
                'version' 	=> IBX_WPFOMO_CONVERTKIT_VER, // current version number
                'license' 	=> $license_key, // license key
                'item_id'   => 54314, // id of this addon
    			'item_name' => 'ConvertKit Addon', 	// name of this addon
    			'author' 	=> 'IdeaBox Creations',  // author of this addon
    			'beta'		=> false
    		)
    	);
	}

	/** 
	 * Updates the meta for the new fields introduced in WPfomify 2.0
	 * 
	 * @since 1.0.2
	 */
	public function update_meta( $data )
	{
		$data['convertkit'] = array(
			'condition'	=> array(
				'ibx_wpfomo_type'				=> 'conversion',
				'ibx_wpfomo_conversions_source'	=> 'convertkit'
			),
			'fields'	=> array(
				'ibx_wpfomo_convertkit_limit' => array( 'ibx_wpfomo_display_last' )
			)
		);

		return $data;
	}
	
	/**
     * Render error notices.
     *
     * @since 1.0.0
     */
    public function admin_notices()
    {
        if ( ! is_admin() ) {
			return;
		}
		else if ( ! is_user_logged_in() ) {
			return;
		}
		else if ( ! current_user_can( 'update_core' ) ) {
			return;
		}

        if ( count( $this->errors ) ) {
			foreach ( $this->errors as $key => $msg ) {
				?>
				<div class="notice notice-error">
					<p><?php echo $msg; ?></p>
				</div>
				<?php
			}
		}

		if ( class_exists( 'IBX_WPFomo_Helper' ) ) {
			// displays saved data for debug purpose
			// when saved_log param is passed in URL.
			IBX_WPFomo_Helper::display_saved_log( $this->meta_key );
		}
    }

    /**
     * Init fields for ConvertKit integration.
     *
     * @since 1.0.0
     * @return void
     */
    public function init_fields()
    {
        add_filter( 'ibx_wpfomo_field_fomo_type', array( $this, 'hide_fields' ), 10, 1 );
        add_filter( 'ibx_wpfomo_field_conversions_source', array( $this, 'conversion_source' ), 10, 1 );
        add_filter( 'ibx_wpfomo_metabox_fields', array( $this, 'add_fields' ), 10, 1 );
    }

    /**
     * Enqueue style and scripts.
     *
     * @since 1.0.0
     * @return void
     */
    public function enqueue_scripts()
    {
        wp_enqueue_style( 'ibx-wpfomo-convertkit-style', IBX_WPFOMO_CONVERTKIT_URL . 'assets/css/main.css' );
        wp_enqueue_script( 'ibx-wpfomo-convertkit-script', IBX_WPFOMO_CONVERTKIT_URL . 'assets/js/main.js', array('jquery'), true );
        wp_localize_script( 'ibx-wpfomo-convertkit-script', 'wpfomo_convertkit', array(
            'messages'  => array(
                'connect_success'   => __('Connected successfully! Forms data has been fetched.', 'ibx-wpfomo')
            )
        ) );
    }

    /**
     * Register admin settings.
     *
     * @since 1.0.0
     * @param array $settings
     * @return array
     */
    public function register_admin_settings( $settings )
    {
        $settings['sections']['convertkit_api_key'] = array(
            'title'     => 'ConvertKit',
            'fields'    => array(
                'convertkit_api_key' => array(
                    'type'              => 'text',
                    'label'             => __('API Key', 'ibx-wpfomo'),
                    'default'           => '',
                    'help'              => __( 'Your API key can be found in your ConvertKit account under Account > Account Info > API Key.', 'ibx-wpfomo' )
                ),
                'convertkit_api_secret' => array(
                    'type'              => 'text',
                    'label'             => __('API Secret', 'ibx-wpfomo'),
                    'default'           => '',
                    'help'              => __( 'Enter your API Secret and press "Connect" below. Your API Secret can be found in your ConvertKit account under Account > Account Info > API Secret.', 'ibx-wpfomo' )
                ),
                'convertkit_connect' => array(
                    'type'              => 'button',
                    'label'             => __('Connect', 'ibx-wpfomo'),
                    'class'             => 'ibx-wpfomo-convertkit-connect'
                ),
            )
        );

        $forms = get_option( 'ibx_wpfomo_convertkit_forms' );

        if ( $forms && ! empty( $forms ) ) {
            $settings['sections']['convertkit_api_key']['fields']['convertkit_connect']['label'] = __('Refresh Forms', 'ibx-wpfomo');
        }

        return $settings;
    }

    /**
     * Connect convertkit via AJAX request.
     *
     * @since 1.0.0
     * @return void
     */
    public function connect_convertkit()
    {
        $response = array(
			'error'  => false,
			'html'   => ''
		);


        if ( ! isset( $_POST['api_key'] ) || empty( $_POST['api_key'] ) ) {
            $response['error'] = __('Error: You must provide an API key.', 'ibx-wpfomo');
        }
        elseif ( ! isset( $_POST['api_secret'] ) || empty( $_POST['api_secret'] ) ) {
            $response['error'] = __('Error: You must provide an API Secret.', 'ibx-wpfomo');
        }

        if ( ! $response['error'] ) {

            $api_data = array(
                'api_key'       => $_POST['api_key'],
                'api_secret'    => $_POST['api_secret'],
            );

            $connection = IBX_WPFomo_ConvertKit_Helper::connect( $api_data );

            if ( $connection['error'] ) {
                $response['error'] = $connection['error'];
            }
            else {
                $post_id = isset( $_POST['post_id'] ) ? $_POST['post_id'] : '';
                $form_id = false;

                if ( ! empty( $post_id ) ) {
                    $form_id = get_post_meta( $post_id, 'ibx_wpfomo_convertkit_form', true );
                }

                $api = IBX_WPFomo_ConvertKit_Helper::get_api( $api_data['api_key'] );

                // Lists field html.
        		try {
                    //$forms = $api->get_resources('forms');

                    $forms_data = $api->make_request( 'v3/forms', 'GET', array(
                        'api_key' => $api_data['api_key']
                    ) );

                    $forms = array();

                    if ( is_object( $forms_data ) && isset( $forms_data->forms ) ) {
                        $forms = $forms_data->forms;
                    }

                    if ( ! empty( $post_id ) ) {
                        $response['html'] .= $this->list_field_html( $forms, $form_id );
                    }

                    update_option( 'ibx_wpfomo_convertkit_forms', $forms );
        		}
        		catch ( Exception $e ) {
        			$response['error'] = $e->getMessage();
        		}
            }
        }

        echo json_encode( $response ); die();
    }

    /**
     * HTML to render in list field.
     *
     * @since 1.0.0
     * @return void
     */
    private function list_field_html( $forms, $form_id )
    {
        ob_start();
        ?>
        <option value=""><?php _e('Choose...', 'ibx-wpfomo'); ?></option>

        <?php foreach ( $forms->forms as $form ) { ?>
            <option value="<?php echo $form->id; ?>"<?php echo $form_id == $form->id ? ' selected="selected"' : ''; ?>><?php echo $form->name; ?></option>
		<?php } ?>

        <?php
        return ob_get_clean();
    }

    /**
     * Get lists from saved option.
     *
     * @since 1.0.0
     * @return array
     */
    public function get_list_options()
    {
        $forms = get_option( 'ibx_wpfomo_convertkit_forms' );
        $options = array();

        if ( $forms && ! empty( $forms ) ) {
            foreach ( $forms as $form ) {
                $options[ $form->id ] = $form->name;
            }
        }

        return $options;
    }

    /**
     * ConvertKit conversion fields.
     *
     * @since 1.0.0
     * @return array
     */
    public function get_fields()
    {
        $fields = array(
            'convertkit_form'    => array(
                'type'              => 'select',
                'label'             => __('ConvertKit Form', 'ibx-wpfomo'),
                'default'           => '',
                'options'           => $this->get_list_options()
            ),
			'convertkit_template'	=> array(
				'type'					=> 'template',
				'label'					=> __('Notification Template', 'ibx-wpfomo'),
				'default'				=> array(
					'0'						=> __('{{name}} has signed up for', 'ibx-wpfomo'),
					'1'						=> '{{title}}',
					'2'						=> '{{time}}'
				),
				'variables'				=> array( '{{name}}', '{{title}}', '{{time}}' ),
				'sanitize'				=> false
			),
            'convertkit_link'    => array(
                'type'              => 'text',
                'label'             => __('Link to', 'ibx-wpfomo'),
                'default'           => '',
            )
        );

        return $fields;
    }

    /**
     * Hide fields when reviews is selected.
     *
     * @since 1.0.0
     * @param array $data
     * @return array
     */
    public function hide_fields( $data )
    {
        $fields = $this->get_fields();

        // Hide fields from other field types.
        foreach ( $data['options'] as $option_key => $option_val ) {
            if ( $option_key != 'conversion' ) {
                foreach ( $fields as $key => $field ) {
                    $data['hide'][$option_key]['fields'][] = $key;
                }
            }
        }

        return $data;
    }

    /**
     * Add ConvertKit as conversion source.
     *
     * @since 1.0.0
     * @param array $data
     * @return array
     */
    public function conversion_source( $data )
    {
        $data['options']['convertkit'] = 'ConvertKit';
        $data['toggle']['convertkit']['fields'] = array_keys( $this->get_fields() );

        return $data;
    }

    /**
     * Add fields for ConvertKit conversion.
     *
     * @since 1.0.0
     * @param array $data
     * @return array
     */
    public function add_fields( $data )
    {
        $fields = $this->get_fields();

        foreach ( $fields as $key => $field ) {
            $data['content']['sections']['content_section']['fields'][$key] = $field;
        }

        return $data;
    }

    /**
     * Add conversion content for ConvertKit.
     *
     * @since 1.0.0
     * @param array $data
     * @param object $settings
     * @return array
     */
    public function add_conversion_data( $data, $settings )
    {
        if ( 'convertkit' != $settings->conversions_source ) {
            return $data;
        }

        $members = IBX_WPFomo_Admin::get_post_meta( $settings->post_id, $this->meta_key );

        if ( ! $members || empty( $members ) ) {
            return $data;
        }

        $members = (array)$members;

        if ( ! isset( $members['subscriptions'] ) || empty( $members['subscriptions'] ) ) {
            return $data;
        }

        // Retreive form name.
        $forms = self::get_list_options();

        $form_id = $settings->convertkit_form;
        $form_name = '';

        if ( isset( $forms[$form_id] ) ) {
            $form_name = $forms[$form_id];
        }

        $limit = $settings->display_last;

        if ( empty( $limit ) || ! $limit ) {
            $limit = 100;
        }

        $count = 0;

        foreach ( $members['subscriptions'] as $member ) {

            if ( $count == $limit ) {
                break;
            }

            if ( ! is_object( $member ) || ! isset( $member->id ) ) {
                continue;
            }

            if ( ! isset( $member->subscriber ) ) {
                continue;
            }

            $subscriber = $member->subscriber;

            $subscriber_name = __('Someone', 'ibx-wpfomo');

            if ( isset( $subscriber->first_name ) && ! empty( $subscriber->first_name ) ) {
                $subscriber_name = $subscriber->first_name;

                if ( isset( $subscriber->last_name ) && ! empty( $subscriber->last_name ) ) {
                    $last_name = $subscriber->last_name;
                    $subscriber_name .= ' ' . $last_name[0] . '.';
                }
            }

            // Data to render notification.
            $fields_data = array(
                'title'     => $form_name,
                'name'      => $subscriber_name,
                'email'     => $subscriber->email_address,
                'time'      => ''
            );

            if ( isset( $subscriber->created_at ) && ! empty( $subscriber->created_at ) ) {
				$fields_data['time'] = IBX_WPFomo_Helper::get_timeago_html( $subscriber->created_at );
            }

            // Link
            $fields_data['url'] = esc_url( $settings->convertkit_link );

            $data['fields'][] = $fields_data;

            $count++;
        }

        $data['template'] = $settings->convertkit_template;

        return $data;
    }

    /**
     * Get saved members from post meta.
     *
     * @since 1.0.0
     * @param int $post_id
     * @return array
     */
    public function get_members( $post_id )
    {
        $conversion_source = IBX_WPFomo_Admin::get_post_meta( $post_id, 'conversions_source' );

        // Return if it's not convertkit conversion.
        if ( 'convertkit' != $conversion_source ) {
            return $post_id;
        }

        $members = array();

        // Get list id from post meta.
        $form_id = IBX_WPFomo_Admin::get_post_meta( $post_id, 'convertkit_form' );

        // Get api key.
        $api_key = IBX_WPFomo_Admin::get_settings( 'convertkit_api_key' );

        // Get api secret.
        $api_secret = IBX_WPFomo_Admin::get_settings( 'convertkit_api_secret' );

        // Get limit.
        $limit = IBX_WPFomo_Admin::get_post_meta( $post_id, 'display_last' );

        // Set limit to 100 if empty.
        if ( empty( $limit ) || ! $limit ) {
            $limit = 100;
        }

        // Get response from convertkit.
        $response = IBX_WPFomo_ConvertKit_Helper::get_members( array(
            'api_key'       => $api_key,
            'api_secret'    => $api_secret,
            'form_id'       => $form_id,
            'limit'         => $limit
        ) );

        if ( ! $response['error'] ) {
            $members = $response['members'];
        } else {
            $this->errors[] = $response['error'];
        }

        return $members;
    }

    /**
     * Cron job to fetch mailchimp subscribers.
     *
     * @since 1.0.0
     * @param int $post_id
     * @return void
     */
    public function save_post( $post_id )
    {
        $this->update_data( $post_id );

		IBX_WPFomo_Cron::set_cron( $post_id );
    }

    /**
     * Cron job to fetch mailchimp subscribers.
     *
     * @since 1.0.0
     * @param array $args
     * @return void
     */
    public function update_data( $post_id )
    {
        if ( empty( $post_id ) ) {
            return;
        }

        $members = $this->get_members( $post_id );

        // Update fetched members in post meta.
        IBX_WPFomo_Admin::update_post_meta( $post_id, $this->meta_key, $members );
    }

    /**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 * @return object The IBX_WPFomo_ConvertKit object.
	 */
	public static function get_instance()
	{
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof IBX_WPFomo_ConvertKit ) ) {
			self::$instance = new IBX_WPFomo_ConvertKit();
		}

		return self::$instance;
	}
}

$ibx_wpfomo_convertkit = IBX_WPFomo_ConvertKit::get_instance();
